import {Component, EventEmitter, Output} from '@angular/core';
import {OtherComponent} from "@component/schedule/tabs/other/other.component";
import {MatTab, MatTabChangeEvent, MatTabGroup} from "@angular/material/tabs";
import {map, Observable} from "rxjs";
import {ReactiveFormsModule} from "@angular/forms";
import {MatButton} from "@angular/material/button";
import {DataSpinnerComponent} from "@component/common/data-spinner/data-spinner.component";
import {GroupComponent} from "@component/schedule/tabs/group/group.component";
import {ProfessorComponent} from "@component/schedule/tabs/professor/professor.component";
import {LectureHallComponent} from "@component/schedule/tabs/lecture-hall/lecture-hall.component";
import {ScheduleService} from "@api/v1/schedule.service";
import {ScheduleResponse} from "@api/v1/scheduleResponse";

export enum TabsSelect {
  Group,
  Professor,
  LectureHall,
  Other
}

@Component({
  selector: 'app-tabs',
  standalone: true,
  imports: [
    OtherComponent,
    MatTabGroup,
    MatTab,
    ReactiveFormsModule,
    MatButton,
    DataSpinnerComponent,
    GroupComponent,
    ProfessorComponent,
    LectureHallComponent
  ],
  templateUrl: './tabs.component.html',
  styleUrl: './tabs.component.css',
  providers: [ScheduleService]
})

export class TabsComponent {
  @Output() eventResult = new EventEmitter<[TabsSelect, number, Observable<ScheduleResponse[]>]>();

  constructor(private scheduleApi: ScheduleService) {
  }

  protected campuses: Observable<CampusBasicInfoResponse[]> = of([]);
  protected lectureHalls: Observable<LectureHallResponse[]> = of([]);
  private lectureHallsData: Observable<LectureHallResponse[]> = of([]);
  protected groupSelected(id: number) {
    this.eventResult.emit(
      [
        TabsSelect.Group,
        id,
        this.scheduleApi.getByGroup(id)
          .pipe(
            map(g =>
              g.map(data =>
                ({
                  dayOfWeek: data.dayOfWeek,
                  pairNumber: data.pairNumber,
                  isEven: data.isEven,
                  discipline: data.discipline,
                  disciplineId: data.disciplineId,
                  isExcludedWeeks: data.isExcludedWeeks,
                  weeks: data.weeks,
                  typeOfOccupations: data.typeOfOccupations,
                  group: data.group,
                  groupId: data.groupId,
                  lectureHalls: data.lectureHalls,
                  lectureHallsId: data.lectureHallsId,
                  professors: data.professors,
                  professorsId: data.professorsId,
                  campus: data.campus,
                  campusId: data.campusId,
                  linkToMeet: data.linkToMeet
                }))
            )
          )
      ]
    );
  }

  protected professorSelected(id: number) {
    this.eventResult.emit(
      [
        TabsSelect.Professor,
        id,
        this.scheduleApi.getByProfessor(id)
          .pipe(
            map(p =>
              p.map(data =>
                ({
                  dayOfWeek: data.dayOfWeek,
                  pairNumber: data.pairNumber,
                  isEven: data.isEven,
                  discipline: data.discipline,
                  disciplineId: data.disciplineId,
                  isExcludedWeeks: data.isExcludedWeeks,
                  weeks: data.weeks,
                  typeOfOccupations: data.typeOfOccupations,
                  group: data.group,
                  groupId: data.groupId,
                  lectureHalls: data.lectureHalls,
                  lectureHallsId: data.lectureHallsId,
                  professors: data.professors,
                  professorsId: data.professorsId,
                  campus: data.campus,
                  campusId: data.campusId,
                  linkToMeet: data.linkToMeet
                }))
            )
          )
      ]
    );
  }

  protected lectureHallSelected(id: number) {
    this.eventResult.emit(
      [
        TabsSelect.LectureHall,
        id,
        this.scheduleApi.getByLectureHall(id)
          .pipe(
            map(lh =>
              lh.map(data =>
                ({
                  dayOfWeek: data.dayOfWeek,
                  pairNumber: data.pairNumber,
                  isEven: data.isEven,
                  discipline: data.discipline,
                  disciplineId: data.disciplineId,
                  isExcludedWeeks: data.isExcludedWeeks,
                  weeks: data.weeks,
                  typeOfOccupations: data.typeOfOccupations,
                  group: data.group,
                  groupId: data.groupId,
                  lectureHalls: data.lectureHalls,
                  lectureHallsId: data.lectureHallsId,
                  professors: data.professors,
                  professorsId: data.professorsId,
                  campus: data.campus,
                  campusId: data.campusId,
                  linkToMeet: data.linkToMeet
                }))
            )
          )
      ]
    );
  }

  protected async chooseTabs(event: MatTabChangeEvent) {
    switch (event.index) {
    }
  }

  protected async loadDisciplines() {
    if (!this.disciplinesLoaded) {
      this.disciplineApi.getDisciplines().pipe(
        catchError((error) => {
          this.disciplinesLoaded = null;
          throw error;
        })).subscribe(data => {
        this.disciplineEx.Data = data.map(x =>
          ({
            id: x.id,
            name: x.name,
            selected: false
          }));
        this.disciplinesLoaded = true;
      });
    }
  }

  protected async loadGroups() {
    if (!this.facultiesLoaded)
      await this.facultyLoad();

    if (!this.groupLoaded) {
      this.groupsData = this.groupApi.getGroups();
      this.groupsData.pipe(
        switchMap(groups => this.faculties.pipe(
          map(campuses => {
            return groups.map(x => {
              const faculties = campuses.find(c => c.id === x.facultyId);
              const name = faculties ? faculties.name : '';
              return {
                id: x.id,
                name: `${x.name} (${name})`,
                selected: false
              };
            });
          })
        ))
      ).subscribe(data => {
        this.groupEx.Data = data;
        this.groupLoaded = true;
      });
    }
  }

  protected async extensionLoad() {
    // Lecture Hall
    await this.loadLectureHalls();

    // Disciplines
    await this.loadDisciplines();

    // Groups
    await this.loadGroups();

    // Professors
    if (this.professorsData.length === 0)
      this.professorsLoad();
  }

  @ViewChild('discipline') disciplineEx!: OtherComponent;
  @ViewChild('lecture') lectureHallEx!: OtherComponent;
  @ViewChild('group') groupEx!: OtherComponent;
  @ViewChild('professor') professorEx!: OtherComponent;
}
